﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace VolumeRayCaster
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            bitmap = new Bitmap(ClientSize.Width, ClientSize.Height);
            renderer = new Renderer(this,bitmap,ClientSize.Width,ClientSize.Height);

            World cube = new Cube(new Vector(-0.5,-0.5,-0.5),new Vector(0.5,0.5,0.5));
            World sphere = new Sphere(1);
            World juliaSphere = new Julia3D(new Vector(0.0223, 0.016, 0.01234),10);
            World julia = new Julia3D(new Vector(0.13, 0.24, 0.22), 20);
            World halfspace = new HalfspaceWorld(0,1,0,1);
            World julia2 = new Julia3D(new Vector(0.5, 0.4, 0.3), 20);
            renderer.world = julia;

            /*
            renderer.lights.Add(new Light(new Vector(1,-1,2),new Vector(0.2,0.1,0.05), 2.5));
            renderer.lights.Add(new Light(new Vector(-4, -0.5, 1), new Vector(0.2, 0.5, 0.05),1.5));
            renderer.lights.Add(new Light(new Vector(0, -2, -1), new Vector(0.3, 0.3, 0.4),3.5));
             */

            renderer.lights.Add(new Light(new Vector(1, 1, 2), new Vector(0.3, 0.3, 0.05), 1.5));
            renderer.lights.Add(new Light(new Vector(-4, -0.5, 0), new Vector(0.2, 0.3, 0.05), 2.5));
            renderer.lights.Add(new Light(new Vector(2, -3, -1), new Vector(0.3, 0.3, 0.3), 4.5));

            renderer.showProgress = true;

            renderer.fov = DegToRad.Convert(80);
            renderer.step = 0.005;
            renderer.farClipping = 3;

            renderer.transformation = new TranslationMatrix(0, 0, -2) * ZXYRotationMatrix.New(0.0, DegToRad.Convert(-16), DegToRad.Convert(20));


            // Renderuj klatkę...
            renderThread = new Thread(new ThreadStart(renderer.Render));

            // ...lub renderuj animację i zapisz do pliku
            //renderThread = new Thread(new ThreadStart(renderer.Render360));


            renderThread.Start();
        }
        Bitmap bitmap;
        Thread renderThread;
        Renderer renderer;

        private void MainForm_Paint(object sender, PaintEventArgs e)
        {
            lock (bitmap)
            {
                e.Graphics.DrawImage(bitmap, e.ClipRectangle, e.ClipRectangle, GraphicsUnit.Pixel);
            }
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            renderThread.Abort();
        }
    }
    class DegToRad
    {
        public static double Convert(double deg)
        {
            return deg * Math.PI / 180.0;
        }
    }
}
